#include "sqi.h"

/********************************************
 * sqiTables
 *
 ********************************************/
int sqiTables( HSQISTMT hStmt )
{
	HSQI		hSQI;
	HINI		hIni;
	char		szTableName[INI_MAX_OBJECT_NAME+1];
	char		szDescription[INI_MAX_PROPERTY_VALUE+1];
	char		**pData;

	if ( !hStmt )
		return false;

	sprintf( hStmt->szMsg, "hStmt (%d)", hStmt );
	logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_INFO, SQI_SUCCESS, hStmt->szMsg );

	if ( hStmt->hTableResult )
		sqiFreeResult( hStmt );

	hSQI = (HSQI)hStmt->hSQI;

	hStmt->hTableResult 				= malloc( sizeof(SQITABLE) );
	hStmt->hTableResult->pszTableName	= (char*)strdup( "sqiTables" );
	hStmt->hTableResult->pszFileName	= NULL;
	hStmt->hTableResult->nRefs			= 0;
	hStmt->hTableResult->hBase			= NULL;

	/* table header */
	hStmt->hTableResult->hColHdrs	= lstOpen();
	lstSetFreeFunc( hStmt->hTableResult->hColHdrs, _sqiFreeColumnHeader );

	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 0, "system", "sqiColumns", "TABLE_CAT" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 1, "system", "sqiColumns", "TABLE_SCHEM" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 2, "system", "sqiColumns", "TABLE_NAME" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 3, "system", "sqiColumns", "TABLE_TYPE" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 4, "system", "sqiColumns", "REMARKS" ) );

	/* table rows (based upon root table col hdrs) */
	hStmt->hTableResult->hRows		= lstOpen();
	lstSetFreeFunc( hStmt->hTableResult->hRows, _sqiFreeDataRow );

	if ( hSQI->pszCatalogFile )
	{
		if ( iniOpen( &hIni, hSQI->pszCatalogFile, ';', '[', ']', '=', false ) != INI_SUCCESS )
		{
			sprintf( hStmt->szMsg, "Could not open %s", hSQI->pszCatalogFile );
			logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_INFO, SQI_SUCCESS, hStmt->szMsg );
		}
		else
		{
			iniObjectFirst( hIni );
			while ( !iniObjectEOL( hIni ) )
			{
				szTableName[0]		= '\0';
				szDescription[0]	= '\0';
				iniObject( hIni, szTableName );

				if ( iniPropertySeek( hIni, szTableName, "Description", "" ) == INI_SUCCESS )
					iniValue( hIni, szDescription );

				pData		= malloc( sizeof(char *)*6 );

				pData[0]	= (char *)strdup( hSQI->pszCatalogFile );
				pData[1]	= (char *)strdup( "" );
				pData[2]	= (char *)strdup( szTableName );
				pData[3]	= (char *)strdup( "" );
				pData[4]	= (char *)strdup( szDescription );

				pData[5]	= NULL; /* data terminator... see _sqiFreeDataRow */

				lstAppend( hStmt->hTableResult->hRows, pData );

				iniObjectNext( hIni );
			}
			iniClose( hIni );
		} /* if iniOpen */
	} /* if catalog name */

	hStmt->nRowsAffected	= hStmt->hTableResult->hRows->nItems;

	/* we should be EOL for call to fetch */
	lstLast( hStmt->hTableResult->hRows );
	lstNext( hStmt->hTableResult->hRows );

	return true;
}

